<?php

namespace Drupal\page_cache_vary;

use Drupal\page_cache_vary\StackMiddleware\PageCacheVary;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Parameter;
use Symfony\Component\DependencyInjection\Reference;

/**
 * Instantiate the new Page Cache Vary service.
 *
 * @see \Drupal\Core\Cache\Context\CacheContextsPass
 */
class VaryCacheContextApplyServicePass implements CompilerPassInterface {

  /**
   * Modify the existing http_middleware.page_cache service.
   *
   * Some modules (ex. subrequests,
   * https://git.drupalcode.org/project/subrequests/-/commit/bee4766b196235dada31264921c0e8cc4ad71f4f)
   * requires the http_middleware.page_cache service to be available if the
   * module page_cache is available, and replaces it.
   * Using another service name is, in this case, not an option: we need to
   * replace the service definition by our own.
   */
  public function process(ContainerBuilder $container) {
    $pageCacheDefinition = $container->getDefinition('http_middleware.page_cache');
    $pageCacheDefinition
      ->setClass(PageCacheVary::class)
      ->addArgument(new Reference('service_container'))
      ->addArgument(new Reference('cache.page_vary_metadata'))
      ->addArgument(new Parameter('vary_cache_contexts'));
  }

}
