<?php

namespace Drupal\page_cache_vary;

use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * Heavily inspired by \Drupal\Core\Cache\Context\CacheContextsPass.
 *
 * Retrieve all the CacheContextsPass implementing VaryCacheContextInterface to
 * feed \Drupal\page_cache_vary\StackMiddleware\PageCacheVary.
 *
 * @see \Drupal\Core\Cache\Context\CacheContextsPass
 */
class VaryCacheContextPass implements CompilerPassInterface {

  /**
   * Retrieve all the cache contexts that implements VaryCacheContextInterface.
   *
   * Validation of the cache contexts is already done in the base
   * \Drupal\Core\Cache\Context\CacheContextsPass, so we can consider that the
   * cache contexts we retrieve here are valid, we only need to focus on the
   * interface they implement.
   */
  public function process(ContainerBuilder $container) {
    $cacheContexts = [];

    foreach (array_keys($container->findTaggedServiceIds('cache.context')) as $id) {
      $definition = $container->getDefinition($id);

      if (is_a($definition->getClass(), VaryCacheContextInterface::class, TRUE)) {
        $cacheContexts[] = substr($id, 14);
      }
    }

    $container->setParameter('vary_cache_contexts', $cacheContexts);
  }

}
