<?php

declare(strict_types=1);

namespace Drupal\page_cache_vary_test\Controller;

use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Controller\ControllerBase;
use Drupal\page_cache_vary_test\Cache\Context\CountryCacheContext;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Returns responses for Page cache vary test routes.
 */
final class PageCacheVaryTestController extends ControllerBase {

  public function __construct(protected RequestStack $requestStack, protected ImmutableConfig $config) {
  }

  /**
   * Create a new instance of PageCacheVaryTestController.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('request_stack'),
      $container->get('config.factory')->get('page_cache_vary_test.settings'),
    );
  }

  /**
   * Response varying by country.
   */
  public function withCountry(): array {

    $countryName = match($this->requestStack->getMainRequest()->headers->get(CountryCacheContext::COUNTRY_CODE_HEADER)) {
      'ch' => 'Switzerland',
            'fr' => 'France',
            NULL, '' => 'Unknown',
    };

    $build['content'] = [
      '#type' => 'item',
      '#markup' => $this->t('Country: %country', ['%country' => $countryName]),
      '#cache' => [
        'contexts' => [
          'country',
        ],
      ],
    ];

    return $build;
  }

  /**
   * Response not varying by country.
   */
  public function withoutCountry(): array {

    $build['content'] = [
      '#type' => 'item',
      '#markup' => $this->t('No country here'),
    ];

    return $build;
  }

  /**
   * Return alternatively with country and without country.
   */
  public function alternatingWithCountry(): array {
    $response = $this->config->get('vary') ? $this->withCountry() : $this->withoutCountry();
    $response['#cache']['tags'] = $this->config->getCacheTags();
    return $response;
  }

}
